@@ -19,6 +19,7 @@
 
 package org.apache.wss4j.dom.message;
 
+import java.security.Provider;
 import java.util.ArrayList;
 import java.util.List;
 
@@ -31,6 +32,7 @@ import org.apache.wss4j.common.crypto.Crypto;
 import org.apache.wss4j.common.ext.WSSecurityException;
 import org.apache.wss4j.common.token.Reference;
 import org.apache.wss4j.common.token.SecurityTokenReference;
+import org.apache.wss4j.common.util.FIPSUtils;
 import org.apache.wss4j.common.util.XMLUtils;
 import org.apache.wss4j.dom.WSConstants;
 import org.apache.wss4j.dom.handler.RequestData;
@@ -80,7 +82,8 @@ public class WSSecEncrypt extends WSSecEncryptedKey {
     /**
      * Algorithm to be used with the ephemeral key
      */
-    private String symEncAlgo = WSConstants.AES_128;
+    private String symEncAlgo = FIPSUtils.isFIPSEnabled()
+        ? WSConstants.AES_128_GCM : WSConstants.AES_128;
 
     public WSSecEncrypt(WSSecHeader securityHeader) {
         super(securityHeader);
@@ -90,6 +93,10 @@ public class WSSecEncrypt extends WSSecEncryptedKey {
         super(doc);
     }
 
+    public WSSecEncrypt(Document doc, Provider provider) {
+        super(doc, provider);
+    }
+
     /**
      * Initialize a WSSec Encrypt.
      *
@@ -114,9 +121,8 @@ public class WSSecEncrypt extends WSSecEncryptedKey {
         }
     }
 
-
-    public Document build(Crypto crypto, SecretKey symmetricKey) 
-        throws WSSecurityException {
+    public Document build(Crypto crypto, SecretKey symmetricKey)
+            throws WSSecurityException {
         return this.build(crypto, symmetricKey, null);
     }
     /**
@@ -131,10 +137,10 @@ public class WSSecEncrypt extends WSSecEncryptedKey {
      * @return the SOAP envelope with encrypted Body as <code>Document</code>
      * @throws WSSecurityException
      */
-    public Document build(Crypto crypto, SecretKey symmetricKey, RequestData data)
+    public Document build(Crypto crypto, SecretKey symmetricKey, RequestData reqData)
         throws WSSecurityException {
 
-        prepare(crypto, symmetricKey, data);
+        prepare(crypto, symmetricKey, reqData);
 
         LOG.debug("Beginning Encryption...");
 
