@@ -3,16 +3,19 @@
 -- GovWay Configurazione
 
 -- Gli indici vengono eliminati automaticamente una volta eliminata la tabella
+DROP TRIGGER trg_users_ricerche;
 DROP TRIGGER trg_users_servizi;
 DROP TRIGGER trg_users_soggetti;
 DROP TRIGGER trg_users_password;
 DROP TRIGGER trg_users_stati;
 DROP TRIGGER trg_users;
+DROP TABLE users_ricerche;
 DROP TABLE users_servizi;
 DROP TABLE users_soggetti;
 DROP TABLE users_password;
 DROP TABLE users_stati;
 DROP TABLE users;
+DROP SEQUENCE seq_users_ricerche;
 DROP SEQUENCE seq_users_servizi;
 DROP SEQUENCE seq_users_soggetti;
 DROP SEQUENCE seq_users_password;
@@ -20,21 +23,6 @@ DROP SEQUENCE seq_users_stati;
 DROP SEQUENCE seq_users;
 
 
--- govway
-		  		
--- Gli indici vengono eliminati automaticamente una volta eliminata la tabella
--- DROP INDEX parameters_index;
--- DROP INDEX operations_gdo;
--- DROP INDEX operations_precedenti;
--- DROP INDEX operations_superuser;
-DROP TRIGGER trg_parameters;
-DROP TRIGGER trg_operations;
-DROP TABLE parameters;
-DROP TABLE operations;
-DROP SEQUENCE seq_parameters;
-DROP SEQUENCE seq_operations;
-
-
 -- govway
 		  		
 -- Gli indici vengono eliminati automaticamente una volta eliminata la tabella
