@@ -10,5 +10,7 @@ VAULT_CONFIG=properties
 VAULT_JDBC=jdbc
 
 COMMAND="${JAVA_HOME}/bin/java"
+# per consentire il setEnv via govway.map.properties
+JAVA_OPTS="$JAVA_OPTS --add-opens=java.base/java.util=ALL-UNNAMED"
 
-${COMMAND} -cp "${VAULT_CLASSPATH}/*":"${VAULT_CONFIG}":"${VAULT_JDBC}/*" org.openspcoop2.pdd.config.vault.cli.VaultTools encrypt $*
+${COMMAND} ${JAVA_OPTS} -cp "${VAULT_CLASSPATH}/*":"${VAULT_CONFIG}":"${VAULT_JDBC}/*" org.openspcoop2.pdd.config.vault.cli.VaultTools encrypt $*
