@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2019 Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2023 Oracle and/or its affiliates. All rights reserved.
  *
  * This program and the accompanying materials are made available under the
  * terms of the Eclipse Distribution License v. 1.0, which is available at
@@ -15,11 +15,10 @@ import java.util.*;
 import java.util.logging.Level;
 import java.util.logging.Logger;
 
-import javax.activation.DataHandler;
-import javax.activation.DataSource;
-import javax.xml.soap.*;
+import jakarta.activation.DataHandler;
+import jakarta.activation.DataSource;
+import jakarta.xml.soap.*;
 import javax.xml.stream.XMLStreamReader;
-import javax.xml.transform.Source;
 import javax.xml.transform.stax.StAXSource;
 import javax.xml.transform.stream.StreamSource;
 
@@ -51,7 +50,7 @@ public abstract class MessageImpl
     public static final String CONTENT_ID             = "Content-ID";
     public static final String CONTENT_LOCATION       = "Content-Location";
 
-    protected static final Logger log =
+    private static final Logger log =
         Logger.getLogger(LogDomainConstants.SOAP_DOMAIN,
                          "com.sun.xml.messaging.saaj.soap.LocalStrings");
 
@@ -371,8 +370,8 @@ public abstract class MessageImpl
 
     }
 
-    public MessageImpl(MimeHeaders headers, ContentType ct, int stat,
-            XMLStreamReader reader) throws SOAPExceptionImpl {
+    protected MessageImpl(MimeHeaders headers, ContentType ct, int stat,
+                          XMLStreamReader reader) throws SOAPExceptionImpl {
         init(headers, stat, ct, reader);
     }
 
@@ -409,8 +408,8 @@ public abstract class MessageImpl
                     Level.SEVERE,
                     "SAAJ0533.soap.incorrect.Content-Type",
                     new String[] {
-                        contentType.toString(),
-                        getExpectedContentType()});
+                        contentType.toString().replaceAll("[\r\n]",""),
+                        getExpectedContentType().replaceAll("[\r\n]","")});
                 throw new SOAPVersionMismatchException(
                     "Cannot create message: incorrect content-type for SOAP version. Got: "
                         + contentType
@@ -574,7 +573,7 @@ public abstract class MessageImpl
                   || isSOAPBodyXOPPackage(soapPartCType))) {
                     log.log(Level.SEVERE,
                             "SAAJ0549.soap.part.invalid.Content-Type",
-                            new Object[] {baseType});
+                            new Object[] {baseType.replaceAll("[\r\n]","")});
                     throw new SOAPExceptionImpl(
                             "Bad Content-Type for SOAP Part : " +
                             baseType);
@@ -583,9 +582,9 @@ public abstract class MessageImpl
                 SOAPPart soapPart = getSOAPPart();
                 setMimeHeaders(soapPart, soapMessagePart);
                 soapPart.setContent(isFastInfoset ?
-                     (Source) FastInfosetReflection.FastInfosetSource_new(
-                         soapPartInputStream) :
-                     (Source) new StreamSource(soapPartInputStream));
+                        FastInfosetReflection.FastInfosetSource_new(
+                            soapPartInputStream) :
+                        new StreamSource(soapPartInputStream));
             } else {
                 log.severe("SAAJ0534.soap.unknown.Content-Type");
                 throw new SOAPExceptionImpl("Unrecognized Content-Type");
@@ -619,9 +618,9 @@ public abstract class MessageImpl
         Object lazyParsingProp = getProperty(LAZY_SOAP_BODY_PARSING);
         if (lazyParsingProp == null) return false;
         if (lazyParsingProp instanceof Boolean) {
-            return ((Boolean) lazyParsingProp).booleanValue();
+            return (Boolean) lazyParsingProp;
         } else {
-            return Boolean.valueOf(lazyParsingProp.toString());
+            return Boolean.parseBoolean(lazyParsingProp.toString());
         }
     }
     @Override
@@ -650,7 +649,7 @@ public abstract class MessageImpl
                 }
             } catch (Exception e) {
                 log.log(Level.SEVERE, "SAAJ0591.soap.exception.in.set.property", 
-                    new Object[] {e.getMessage(), "javax.xml.soap.write-xml-declaration"});
+                    new Object[] {e.getMessage().replaceAll("[\r\n]",""), "jakarta.xml.soap.write-xml-declaration"});
                 throw new RuntimeException(e);
             }
             return;
@@ -661,7 +660,7 @@ public abstract class MessageImpl
                 ((EnvelopeImpl) getSOAPPart().getEnvelope()).setCharsetEncoding((String)value);
             } catch (Exception e) {
                 log.log(Level.SEVERE, "SAAJ0591.soap.exception.in.set.property", 
-                    new Object[] {e.getMessage(), "javax.xml.soap.character-set-encoding"});
+                    new Object[] {e.getMessage().replaceAll("[\r\n]",""), "jakarta.xml.soap.character-set-encoding"});
                 throw new RuntimeException(e);
             }
         }
@@ -778,7 +777,7 @@ public abstract class MessageImpl
     }
 
     public void setContentType(String type) {
-        headers.setHeader("Content-Type", type);
+        headers.setHeader("Content-Type", convertToSingleLine(type));
         needsSave();
     }
 
@@ -806,7 +805,7 @@ public abstract class MessageImpl
     public void setBaseType(String type) {
         ContentType ct = contentType();
         ct.setParameter("type", type);
-        headers.setHeader("Content-Type", ct.toString());
+        headers.setHeader("Content-Type", convertToSingleLine(ct.toString()));
         needsSave();
     }
 
@@ -817,7 +816,7 @@ public abstract class MessageImpl
     public void setAction(String action) {
         ContentType ct = contentType();
         ct.setParameter("action", action);
-        headers.setHeader("Content-Type", ct.toString());
+        headers.setHeader("Content-Type", convertToSingleLine(ct.toString()));
         needsSave();
     }
 
@@ -828,7 +827,7 @@ public abstract class MessageImpl
     public void setCharset(String charset) {
         ContentType ct = contentType();
         ct.setParameter("charset", charset);
-        headers.setHeader("Content-Type", ct.toString());
+        headers.setHeader("Content-Type", convertToSingleLine(ct.toString()));
         needsSave();
     }
 
@@ -905,7 +904,7 @@ public abstract class MessageImpl
         needsSave();
     }
 
-    static private final Iterator<AttachmentPart> nullIter = Collections.EMPTY_LIST.<AttachmentPart>iterator();
+    static private final Iterator<AttachmentPart> nullIter = Collections.emptyIterator();
 
     @Override
     public Iterator<AttachmentPart> getAttachments() {
@@ -928,7 +927,7 @@ public abstract class MessageImpl
         ct.setPrimaryType(split[0]);
         ct.setSubType(split[1]);
         ct.setParameter("charset", charset);
-        headers.setHeader("Content-Type", ct.toString());
+        headers.setHeader("Content-Type", convertToSingleLine(ct.toString()));
     }
 
     private class MimeMatchingIterator implements Iterator<AttachmentPart> {
@@ -1005,7 +1004,7 @@ public abstract class MessageImpl
             int index = attachments.indexOf(it.next());
             attachments.set(index, null);
         }
-        FinalArrayList<AttachmentPart> f = new FinalArrayList<AttachmentPart>();
+        FinalArrayList<AttachmentPart> f = new FinalArrayList<>();
         for (int i = 0; i < attachments.size(); i++) {
             if (attachments.get(i) != null) {
                 f.add(attachments.get(i));
@@ -1071,27 +1070,27 @@ public abstract class MessageImpl
                 MimeHeaders headersToMatch = new MimeHeaders();
                 headersToMatch.addHeader(CONTENT_ID, uri);
                 
-                Iterator i = this.getAttachments(headersToMatch);
-                _part = (i == null) ? null : (AttachmentPart)i.next();
+                Iterator<AttachmentPart> i = this.getAttachments(headersToMatch);
+                _part = (i == null) ? null : i.next();
             } else {
                 // try content-location
                 MimeHeaders headersToMatch = new MimeHeaders();
                 headersToMatch.addHeader(CONTENT_LOCATION, uri);
                    
-                Iterator i = this.getAttachments(headersToMatch);
-                _part = (i == null) ? null : (AttachmentPart)i.next();
+                Iterator<AttachmentPart> i = this.getAttachments(headersToMatch);
+                _part = (i == null) ? null : i.next();
             }
 
             // try  auto-generated JAXRPC CID
             if (_part == null) {
-                Iterator j = this.getAttachments();
+                Iterator<AttachmentPart> j = this.getAttachments();
                     
                 while (j.hasNext()) {
-                    AttachmentPart p = (AttachmentPart)j.next();
+                    AttachmentPart p = j.next();
                     String cl = p.getContentId();
                     if (cl != null) {    
                         // obtain the partname
-                        int eqIndex = cl.indexOf("=");
+                        int eqIndex = cl.indexOf('=');
                         if (eqIndex > -1) {
                             cl = cl.substring(1, eqIndex);
                             if (cl.equalsIgnoreCase(uri)) {
@@ -1104,7 +1103,7 @@ public abstract class MessageImpl
             }
             
         } catch (Exception se) {
-            log.log(Level.SEVERE, "SAAJ0590.soap.unable.to.locate.attachment", new Object[] {uri});
+            log.log(Level.SEVERE, "SAAJ0590.soap.unable.to.locate.attachment", new Object[] {uri.replaceAll("[\r\n]","")});
             throw new SOAPExceptionImpl(se);
         }
         return _part;
@@ -1170,7 +1169,7 @@ public abstract class MessageImpl
                 headerAndBody = new MimeMultipart();
                 headerAndBody.addBodyPart(mimeSoapPart);
 
-                for (Iterator eachAttachement = getAttachments();
+                for (Iterator<AttachmentPart> eachAttachement = getAttachments();
                     eachAttachement.hasNext();
                     ) {
                     headerAndBody.addBodyPart(
@@ -1228,7 +1227,7 @@ public abstract class MessageImpl
 
     private String getCharsetString(String s) {
         try {
-            int index = s.indexOf(";");
+            int index = s.indexOf(';');
             if(index < 0)
                 return null;
             ParameterList pl = new ParameterList(s.substring(index));
@@ -1336,7 +1335,7 @@ public abstract class MessageImpl
             mimeSoapPart.setHeader("Content-Type", soapPartCtype.toString());
             headerAndBody.addBodyPart(mimeSoapPart);
                                                                                                                                 
-            for (Iterator eachAttachement = getAttachments();
+            for (Iterator<AttachmentPart> eachAttachement = getAttachments();
                 eachAttachement.hasNext();
                 ) {
                 headerAndBody.addBodyPart(
@@ -1479,7 +1478,7 @@ public abstract class MessageImpl
         }
                                                                                 
         if (attachments == null)
-            attachments = new FinalArrayList<AttachmentPart>();
+            attachments = new FinalArrayList<>();
                                                                                 
         int count = multiPart.getCount();
         for (int i=0; i < count; i++ ) {
@@ -1517,10 +1516,10 @@ public abstract class MessageImpl
         // first remove the existing content-type
         soapPart.removeAllMimeHeaders();
         // add everything present in soapMessagePart
-        List headers = soapMessagePart.getAllHeaders();
+        List<Header> headers = soapMessagePart.getAllHeaders();
         int sz = headers.size();
         for( int i=0; i<sz; i++ ) {
-            Header h = (Header) headers.get(i);
+            Header h = headers.get(i);
             soapPart.addMimeHeader(h.getName(), h.getValue());
         }
     }
