@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2020 Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2022 Oracle and/or its affiliates. All rights reserved.
  *
  * This program and the accompanying materials are made available under the
  * terms of the Eclipse Distribution License v. 1.0, which is available at
@@ -8,10 +8,6 @@
  * SPDX-License-Identifier: BSD-3-Clause
  */
 
-/**
-*
-* @author SAAJ RI Development Team
-*/
 package com.sun.xml.messaging.saaj.soap;
 
 import com.sun.xml.messaging.saaj.soap.impl.CDATAImpl;
@@ -43,21 +39,25 @@ import org.w3c.dom.ProcessingInstruction;
 import org.w3c.dom.Text;
 import org.w3c.dom.UserDataHandler;
 
+import javax.xml.XMLConstants;
 import javax.xml.parsers.DocumentBuilder;
 import javax.xml.parsers.DocumentBuilderFactory;
 import javax.xml.parsers.ParserConfigurationException;
-import javax.xml.soap.SOAPElement;
-import javax.xml.soap.SOAPException;
+import jakarta.xml.soap.SOAPElement;
+import jakarta.xml.soap.SOAPException;
 import java.lang.reflect.Constructor;
 import java.text.MessageFormat;
 import java.util.logging.Logger;
 
-public class SOAPDocumentImpl implements SOAPDocument, javax.xml.soap.Node, Document {
+/**
+ *
+ * @author SAAJ RI Development Team
+ */
+public class SOAPDocumentImpl implements SOAPDocument, jakarta.xml.soap.Node, Document {
 
-    public static final String SAAJ_NODE = "javax.xml.soap.Node";
+    public static final String SAAJ_NODE = "jakarta.xml.soap.Node";
 
-    private static final String XMLNS = "xmlns".intern();
-    protected static final Logger log =
+    private static final Logger log =
         Logger.getLogger(LogDomainConstants.SOAP_DOMAIN,
                          "com.sun.xml.messaging.saaj.soap.LocalStrings");
     
@@ -217,14 +217,14 @@ public class SOAPDocumentImpl implements SOAPDocument, javax.xml.soap.Node, Docu
 
     @Override
     public Attr createAttribute(String name) throws DOMException {
-        boolean isQualifiedName = (name.indexOf(":") > 0);
+        boolean isQualifiedName = (name.indexOf(':') > 0);
         if (isQualifiedName) {
             String nsUri = null;
-            String prefix = name.substring(0, name.indexOf(":"));
+            String prefix = name.substring(0, name.indexOf(':'));
             //cannot do anything to resolve the URI if prefix is not
             //XMLNS.
-            if (XMLNS.equals(prefix)) {
-                nsUri = ElementImpl.XMLNS_URI;
+            if (XMLConstants.XMLNS_ATTRIBUTE.equals(prefix)) {
+                nsUri = XMLConstants.XMLNS_ATTRIBUTE_NS_URI;
                 return createAttributeNS(nsUri, name);
             }
         }
@@ -250,7 +250,7 @@ public class SOAPDocumentImpl implements SOAPDocument, javax.xml.soap.Node, Docu
         Node domNode = getDomNode(importedNode);
         final Node newNode = document.importNode(domNode, deep);
 
-        if (importedNode instanceof javax.xml.soap.Node) {
+        if (importedNode instanceof jakarta.xml.soap.Node) {
             Node newSoapNode = createSoapNode(importedNode.getClass(), newNode);
             newNode.setUserData(SAAJ_NODE, newSoapNode, null);
             if (deep && newSoapNode.hasChildNodes()) {
@@ -595,7 +595,7 @@ public class SOAPDocumentImpl implements SOAPDocument, javax.xml.soap.Node, Docu
     }
 
     /**
-     * Insert a mapping information for {@link org.w3c.dom.Node} - {@link javax.xml.soap.Node}.
+     * Insert a mapping information for {@link org.w3c.dom.Node} - {@link jakarta.xml.soap.Node}.
      *
      * In SAAJ, elements in DOM are expected to be interfaces of SAAJ, on the other hand in JDKs Xerces,
      * they are casted to internal impl classes. After removal of SAAJ dependency
@@ -603,7 +603,7 @@ public class SOAPDocumentImpl implements SOAPDocument, javax.xml.soap.Node, Docu
      *
      * @param node SAAJ wrapper node for w3c DOM node
      */
-    public void register(javax.xml.soap.Node node) {
+    public void register(jakarta.xml.soap.Node node) {
         final Node domElement = getDomNode(node);
         if (domElement.getUserData(SAAJ_NODE) != null) {
             throw new IllegalStateException("Element " + domElement.getNodeName()
@@ -618,18 +618,18 @@ public class SOAPDocumentImpl implements SOAPDocument, javax.xml.soap.Node, Docu
      * @param node w3c dom node nullable
      * @return soap wrapper for w3c dom node
      */
-    public javax.xml.soap.Node find(Node node) {
+    public jakarta.xml.soap.Node find(Node node) {
         return find(node, true);
     }
 
-    private javax.xml.soap.Node find(Node node, boolean required) {
+    private jakarta.xml.soap.Node find(Node node, boolean required) {
         if (node == null) {
             return null;
         }
-        if (node instanceof javax.xml.soap.Node) {
-            return (javax.xml.soap.Node) node;
+        if (node instanceof jakarta.xml.soap.Node) {
+            return (jakarta.xml.soap.Node) node;
         }
-        final javax.xml.soap.Node found = (javax.xml.soap.Node) node.getUserData(SAAJ_NODE);
+        final jakarta.xml.soap.Node found = (jakarta.xml.soap.Node) node.getUserData(SAAJ_NODE);
         if (found == null && required) {
             throw new IllegalArgumentException(MessageFormat.format("Cannot find SOAP wrapper for element {0}", node));
         }
@@ -644,7 +644,7 @@ public class SOAPDocumentImpl implements SOAPDocument, javax.xml.soap.Node, Docu
      * @return soap wrapper or passed w3c dom node if not found
      */
     public Node findIfPresent(Node node) {
-        final javax.xml.soap.Node found = find(node, false);
+        final jakarta.xml.soap.Node found = find(node, false);
         return found != null ? found : node;
     }
 
@@ -672,7 +672,7 @@ public class SOAPDocumentImpl implements SOAPDocument, javax.xml.soap.Node, Docu
     }
 
 
-    private Node createSoapNode(Class nodeType, Node node) {
+    private Node createSoapNode(Class<? extends Node> nodeType, Node node) {
         if (SOAPTextImpl.class.isAssignableFrom(nodeType)) {
             return new SOAPTextImpl(this, (Text) node);
         } else if (SOAPCommentImpl.class.isAssignableFrom(nodeType)) {
@@ -683,7 +683,7 @@ public class SOAPDocumentImpl implements SOAPDocument, javax.xml.soap.Node, Docu
             return new SOAPDocumentFragment(this, (DocumentFragment) node);
         }
         try {
-            Constructor<Node> constructor = nodeType.getConstructor(SOAPDocumentImpl.class, Element.class);
+            Constructor<? extends Node> constructor = nodeType.getConstructor(SOAPDocumentImpl.class, Element.class);
             return constructor.newInstance(this, node);
         } catch (Exception e) {
             throw new IllegalStateException(e);
