@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2020 Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2023 Oracle and/or its affiliates. All rights reserved.
  *
  * This program and the accompanying materials are made available under the
  * terms of the Eclipse Distribution License v. 1.0, which is available at
@@ -16,11 +16,11 @@ import com.sun.xml.messaging.saaj.soap.SOAPDocumentImpl;
 import com.sun.xml.messaging.saaj.soap.name.NameImpl;
 import com.sun.xml.messaging.saaj.util.LogDomainConstants;
 import com.sun.xml.messaging.saaj.util.NamespaceContextIterator;
-import javax.xml.soap.Name;
-import javax.xml.soap.SOAPBodyElement;
-import javax.xml.soap.SOAPConstants;
-import javax.xml.soap.SOAPElement;
-import javax.xml.soap.SOAPException;
+import jakarta.xml.soap.Name;
+import jakarta.xml.soap.SOAPBodyElement;
+import jakarta.xml.soap.SOAPConstants;
+import jakarta.xml.soap.SOAPElement;
+import jakarta.xml.soap.SOAPException;
 import org.w3c.dom.Attr;
 import org.w3c.dom.CharacterData;
 import org.w3c.dom.DOMException;
@@ -33,6 +33,8 @@ import org.w3c.dom.NodeList;
 import org.w3c.dom.Text;
 import org.w3c.dom.TypeInfo;
 import org.w3c.dom.UserDataHandler;
+
+import javax.xml.XMLConstants;
 import javax.xml.namespace.QName;
 import java.net.URI;
 import java.net.URISyntaxException;
@@ -42,13 +44,11 @@ import java.util.NoSuchElementException;
 import java.util.logging.Level;
 import java.util.logging.Logger;
 
-import static com.sun.xml.messaging.saaj.soap.SOAPDocumentImpl.SAAJ_NODE;
-
 public class ElementImpl implements SOAPElement, SOAPBodyElement {
 
-    public static final String DSIG_NS = "http://www.w3.org/2000/09/xmldsig#".intern();
-    public static final String XENC_NS = "http://www.w3.org/2001/04/xmlenc#".intern();
-    public static final String WSU_NS = "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd".intern();
+    public static final String DSIG_NS = "http://www.w3.org/2000/09/xmldsig#";
+    public static final String XENC_NS = "http://www.w3.org/2001/04/xmlenc#";
+    public static final String WSU_NS = "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd";
 
     private transient AttributeManager encodingStyleAttribute = new AttributeManager();
 
@@ -70,16 +70,16 @@ public class ElementImpl implements SOAPElement, SOAPBodyElement {
 
     @Override
     public void setAttribute(String name, String value) throws DOMException {
-        boolean isQualifiedName = (name.indexOf(":") > 0);
+        boolean isQualifiedName = (name.indexOf(':') > 0);
         //this is because of BugfixTest#testCR7020991, after removal internal dependencies
         //SOAPDocumentImpl#createAttribute is not called anymore from xerces parent
         if (isQualifiedName) {
             String nsUri = null;
-            String prefix = name.substring(0, name.indexOf(":"));
+            String prefix = name.substring(0, name.indexOf(':'));
             //cannot do anything to resolve the URI if prefix is not
             //XMLNS.
-            if (XMLNS.equals(prefix)) {
-                nsUri = ElementImpl.XMLNS_URI;
+            if (XMLConstants.XMLNS_ATTRIBUTE.equals(prefix)) {
+                nsUri = XMLConstants.XMLNS_ATTRIBUTE_NS_URI;
                 setAttributeNS(nsUri, name, value);
                 return;
             }
@@ -124,25 +124,8 @@ public class ElementImpl implements SOAPElement, SOAPBodyElement {
         return element.getAttributeNS(namespaceURI, localName);
     }
 
-    protected static final Logger log =
-        Logger.getLogger(LogDomainConstants.SOAP_IMPL_DOMAIN,
-                         "com.sun.xml.messaging.saaj.soap.impl.LocalStrings");
+    private static final Logger log = Logger.getLogger(LogDomainConstants.SOAP_IMPL_DOMAIN, "com.sun.xml.messaging.saaj.soap.impl.LocalStrings");
 
-    /**
-     * XML Information Set REC
-     * all namespace attributes (including those named xmlns, 
-     * whose [prefix] property has no value) have a namespace URI of http://www.w3.org/2000/xmlns/
-     */
-    public final static String XMLNS_URI = "http://www.w3.org/2000/xmlns/".intern();
-    
-    /**
-     * The XML Namespace ("http://www.w3.org/XML/1998/namespace"). This is
-     * the Namespace URI that is automatically mapped to the "xml" prefix.
-     */
-    public final static String XML_URI = "http://www.w3.org/XML/1998/namespace".intern();
-
-    private final static String XMLNS = "xmlns".intern();
-    
     public ElementImpl(SOAPDocumentImpl ownerDoc, Name name) {
         this.soapDocument = ownerDoc;
         this.element = ownerDoc.getDomDocument().createElementNS(name.getURI(), name.getQualifiedName());
@@ -374,7 +357,7 @@ public class ElementImpl implements SOAPElement, SOAPBodyElement {
             log.log( 
                 Level.SEVERE,
                 "SAAJ0101.impl.parent.of.body.elem.mustbe.body",
-                new String[] { prefix });
+                new String[] { prefix.replaceAll("[\r\n]","") });
             throw new SOAPExceptionImpl(
                 "Unable to locate namespace for prefix " + prefix);
         }
@@ -384,12 +367,12 @@ public class ElementImpl implements SOAPElement, SOAPBodyElement {
     @Override
     public String getNamespaceURI(String prefix) {
 
-        if ("xmlns".equals(prefix)) {
-            return XMLNS_URI;
+        if (XMLConstants.XMLNS_ATTRIBUTE.equals(prefix)) {
+            return XMLConstants.XMLNS_ATTRIBUTE_NS_URI;
         }
         
         if("xml".equals(prefix)) {
-            return XML_URI;
+            return XMLConstants.XML_NS_URI;
         }
 
         if ("".equals(prefix)) {
@@ -411,11 +394,11 @@ public class ElementImpl implements SOAPElement, SOAPBodyElement {
                         }
                     }*/
                     if (((Element) currentAncestor).hasAttributeNS(
-                            XMLNS_URI, "xmlns")) {
+                            XMLConstants.XMLNS_ATTRIBUTE_NS_URI, XMLConstants.XMLNS_ATTRIBUTE)) {
 
                         String uri =
                             ((Element) currentAncestor).getAttributeNS(
-                                XMLNS_URI, "xmlns");
+                                XMLConstants.XMLNS_ATTRIBUTE_NS_URI, XMLConstants.XMLNS_ATTRIBUTE);
                         if ("".equals(uri))
                             return null;
                         else {
@@ -448,9 +431,9 @@ public class ElementImpl implements SOAPElement, SOAPBodyElement {
                 //}
                 
                 if (((Element) currentAncestor).hasAttributeNS(
-                        XMLNS_URI, prefix)) {
+                        XMLConstants.XMLNS_ATTRIBUTE_NS_URI, prefix)) {
                     return ((Element) currentAncestor).getAttributeNS(
-                               XMLNS_URI, prefix);
+                               XMLConstants.XMLNS_ATTRIBUTE_NS_URI, prefix);
                 }
 
                 currentAncestor = currentAncestor.getParentNode();
@@ -473,7 +456,7 @@ public class ElementImpl implements SOAPElement, SOAPBodyElement {
         String uri = getNamespaceURI(prefix);
         if (uri == null) {
             log.log(Level.SEVERE, "SAAJ0102.impl.cannot.locate.ns",
-                    new Object[] {prefix});
+                    new Object[] {prefix.replaceAll("[\r\n]","")});
             throw new SOAPException("Unable to locate namespace for prefix " 
                                     + prefix);
         }
@@ -487,7 +470,7 @@ public class ElementImpl implements SOAPElement, SOAPBodyElement {
             org.w3c.dom.Attr namespaceDecl = eachNamespace.nextNamespaceAttr();
             if (namespaceDecl.getNodeValue().equals(uri)) {
                 String candidatePrefix = namespaceDecl.getLocalName();
-                if ("xmlns".equals(candidatePrefix))
+                if (XMLConstants.XMLNS_ATTRIBUTE.equals(candidatePrefix))
                     return "";
                 else
                     return candidatePrefix;
@@ -517,7 +500,7 @@ public class ElementImpl implements SOAPElement, SOAPBodyElement {
                 if (namespaceDecl.getNodeName().endsWith(prefix))
                     return namespaceDecl;
             } else {
-                if (namespaceDecl.getNodeName().equals("xmlns"))
+                if (XMLConstants.XMLNS_ATTRIBUTE.equals(namespaceDecl.getNodeName()))
                     return namespaceDecl;
             }
         }
@@ -580,7 +563,7 @@ public class ElementImpl implements SOAPElement, SOAPBodyElement {
                    log.severe("SAAJ0158.impl.version.mismatch.fault");
                    throw new SOAPExceptionImpl("SOAP Version mismatch encountered when trying to add SOAPFault to SOAPBody");
                }
-               Iterator<javax.xml.soap.Node> it = this.getChildElements();
+               Iterator<jakarta.xml.soap.Node> it = this.getChildElements();
                if (it.hasNext()) {
                    log.severe("SAAJ0156.impl.adding.fault.error");
                    throw new SOAPExceptionImpl("Cannot add SOAPFault as a child of a non-Empty SOAPBody");
@@ -712,10 +695,11 @@ public class ElementImpl implements SOAPElement, SOAPBodyElement {
     
     @Override
     public SOAPElement addTextNode(String text) throws SOAPException {
-        if (text.startsWith(CDATAImpl.cdataUC)
-            || text.startsWith(CDATAImpl.cdataLC))
+        if ((text.startsWith(CDATAImpl.cdataUC) || text.startsWith(CDATAImpl.cdataLC))
+            && text.endsWith(CDATAImpl.CDATA_CLOSE))
             return addCDATA(
-                text.substring(CDATAImpl.cdataUC.length(), text.length() - 3));
+                text.substring(CDATAImpl.cdataUC.length(), 
+                text.length() - CDATAImpl.CDATA_CLOSE.length()));
         return addText(text);
     }
 
@@ -775,8 +759,8 @@ public class ElementImpl implements SOAPElement, SOAPBodyElement {
         String value) {
 
         uri = uri.length() == 0 ? null : uri;
-        if (qualifiedName.equals("xmlns")) {
-            uri = XMLNS_URI;
+        if (qualifiedName.equals(XMLConstants.XMLNS_ATTRIBUTE)) {
+            uri = XMLConstants.XMLNS_ATTRIBUTE_NS_URI;
         }
         
         if (uri == null) {
@@ -790,9 +774,9 @@ public class ElementImpl implements SOAPElement, SOAPBodyElement {
     public SOAPElement addNamespaceDeclaration(String prefix, String uri)
         throws SOAPException {
         if (prefix.length() > 0) {
-            setAttributeNS(XMLNS_URI, "xmlns:" + prefix, uri);
+            setAttributeNS(XMLConstants.XMLNS_ATTRIBUTE_NS_URI, XMLConstants.XMLNS_ATTRIBUTE + ":" + prefix, uri);
         } else {
-            setAttributeNS(XMLNS_URI, "xmlns", uri);
+            setAttributeNS(XMLConstants.XMLNS_ATTRIBUTE_NS_URI, XMLConstants.XMLNS_ATTRIBUTE, uri);
         }
         //Fix for CR:6474641
         //tryToFindEncodingStyleAttributeName();
@@ -820,7 +804,7 @@ public class ElementImpl implements SOAPElement, SOAPBodyElement {
         ArrayList<Name> list = new ArrayList<>();
         while (i.hasNext()) {
             Name name = i.next();
-            if (!"xmlns".equalsIgnoreCase(name.getPrefix()))
+            if (!XMLConstants.XMLNS_ATTRIBUTE.equalsIgnoreCase(name.getPrefix()))
                 list.add(name);
         }
         return list.iterator();
@@ -832,7 +816,7 @@ public class ElementImpl implements SOAPElement, SOAPBodyElement {
         ArrayList<QName> list = new ArrayList<>();
         while (i.hasNext()) {
             Name name = i.next();
-            if (!"xmlns".equalsIgnoreCase(name.getPrefix())) {
+            if (!XMLConstants.XMLNS_ATTRIBUTE.equalsIgnoreCase(name.getPrefix())) {
                 list.add(NameImpl.convertToQName(name));
             }
         }
@@ -861,8 +845,8 @@ public class ElementImpl implements SOAPElement, SOAPBodyElement {
                 while (next == null && eachNamespace.hasNext()) {
                     String attributeKey =
                         eachNamespace.nextNamespaceAttr().getNodeName();
-                    if (attributeKey.startsWith("xmlns:")) {
-                        next = attributeKey.substring("xmlns:".length());
+                    if (attributeKey.startsWith(XMLConstants.XMLNS_ATTRIBUTE + ":")) {
+                        next = attributeKey.substring((XMLConstants.XMLNS_ATTRIBUTE + ":").length());
                     }
                 }
             }
@@ -944,7 +928,7 @@ public class ElementImpl implements SOAPElement, SOAPBodyElement {
     }
 
     @Override
-    public Iterator<javax.xml.soap.Node> getChildElements() {
+    public Iterator<jakarta.xml.soap.Node> getChildElements() {
         return getChildElementsFrom(this);
     }
 
@@ -1067,7 +1051,7 @@ public class ElementImpl implements SOAPElement, SOAPBodyElement {
 
     protected Iterator<Node> getChildElementNodes() {
         return new Iterator<Node>() {
-            Iterator<javax.xml.soap.Node> eachNode = getChildElements();
+            Iterator<jakarta.xml.soap.Node> eachNode = getChildElements();
             Node next = null;
             Node last = null;
 
@@ -1086,11 +1070,11 @@ public class ElementImpl implements SOAPElement, SOAPBodyElement {
             }
 
             @Override
-            public javax.xml.soap.Node next() {
+            public jakarta.xml.soap.Node next() {
                 if (hasNext()) {
                     last = next;
                     next = null;
-                    return (javax.xml.soap.Node) last;
+                    return (jakarta.xml.soap.Node) last;
                 }
                 throw new NoSuchElementException();
             }
@@ -1108,17 +1092,17 @@ public class ElementImpl implements SOAPElement, SOAPBodyElement {
     }
 
     @Override
-    public Iterator<javax.xml.soap.Node> getChildElements(final Name name) {
+    public Iterator<jakarta.xml.soap.Node> getChildElements(final Name name) {
        return getChildElements(name.getURI(), name.getLocalName());
     }
 
     @Override
-    public Iterator<javax.xml.soap.Node> getChildElements(final QName qname) {
+    public Iterator<jakarta.xml.soap.Node> getChildElements(final QName qname) {
         return getChildElements(qname.getNamespaceURI(), qname.getLocalPart());
     }
     
-    private Iterator<javax.xml.soap.Node> getChildElements(final String nameUri, final String nameLocal) {
-        return new Iterator<javax.xml.soap.Node>() {
+    private Iterator<jakarta.xml.soap.Node> getChildElements(final String nameUri, final String nameLocal) {
+        return new Iterator<jakarta.xml.soap.Node>() {
             Iterator<Node> eachElement = getChildElementNodes();
             Node next = null;
             Node last = null;
@@ -1142,13 +1126,13 @@ public class ElementImpl implements SOAPElement, SOAPBodyElement {
             }
 
             @Override
-            public javax.xml.soap.Node next() {
+            public jakarta.xml.soap.Node next() {
                 if (!hasNext()) {
                     throw new NoSuchElementException();
                 }
                 last = next;
                 next = null;
-                return (javax.xml.soap.Node) last;
+                return (jakarta.xml.soap.Node) last;
             }
 
             @Override
@@ -1169,8 +1153,8 @@ public class ElementImpl implements SOAPElement, SOAPBodyElement {
 
         while (currentChild != null) {
             Node temp = currentChild.getNextSibling();
-            if (currentChild instanceof javax.xml.soap.Node) {
-                ((javax.xml.soap.Node) currentChild).detachNode();
+            if (currentChild instanceof jakarta.xml.soap.Node) {
+                ((jakarta.xml.soap.Node) currentChild).detachNode();
             } else {
                 Node parent = currentChild.getParentNode();
                 if (parent != null) {
@@ -1191,7 +1175,7 @@ public class ElementImpl implements SOAPElement, SOAPBodyElement {
                 log.log(
                     Level.SEVERE,
                     "SAAJ0105.impl.encoding.style.mustbe.valid.URI",
-                    new String[] { encodingStyle });
+                    new String[] { encodingStyle.replaceAll("[\r\n]","") });
                 throw new IllegalArgumentException(
                     "Encoding style (" + encodingStyle + ") should be a valid URI");
             }
@@ -1224,7 +1208,7 @@ public class ElementImpl implements SOAPElement, SOAPBodyElement {
     // Node methods
     @Override
     public String getValue() {
-        javax.xml.soap.Node valueNode = getValueNode();
+        jakarta.xml.soap.Node valueNode = getValueNode();
         return valueNode == null ? null : valueNode.getValue();
     }
 
@@ -1257,8 +1241,8 @@ public class ElementImpl implements SOAPElement, SOAPBodyElement {
         return null;
     }
 
-    protected javax.xml.soap.Node getValueNode() {
-        Iterator<javax.xml.soap.Node> i = getChildElements();
+    protected jakarta.xml.soap.Node getValueNode() {
+        Iterator<jakarta.xml.soap.Node> i = getChildElements();
         while (i.hasNext()) {
             Node n = i.next();
             if (n.getNodeType() == org.w3c.dom.Node.TEXT_NODE ||
@@ -1504,8 +1488,8 @@ public class ElementImpl implements SOAPElement, SOAPBodyElement {
         return attribute == null ? null : attribute.getValue();
     }
 
-    protected Iterator<javax.xml.soap.Node> getChildElementsFrom(final Element element) {
-        return new Iterator<javax.xml.soap.Node>() {
+    protected Iterator<jakarta.xml.soap.Node> getChildElementsFrom(final Element element) {
+        return new Iterator<jakarta.xml.soap.Node>() {
             Node next = element.getFirstChild();
             Node nextNext = null;
             Node last = null;
@@ -1524,7 +1508,7 @@ public class ElementImpl implements SOAPElement, SOAPBodyElement {
             }
 
             @Override
-            public javax.xml.soap.Node next() {
+            public jakarta.xml.soap.Node next() {
                 if (hasNext()) {
                     last = next;
                     next = null;
@@ -1537,7 +1521,7 @@ public class ElementImpl implements SOAPElement, SOAPBodyElement {
                     }
 
                     nextNext = last.getNextSibling();
-                    return (javax.xml.soap.Node) soapDocument.findIfPresent(last);
+                    return (jakarta.xml.soap.Node) soapDocument.findIfPresent(last);
                 }
                 throw new NoSuchElementException();
             }
@@ -1664,13 +1648,13 @@ public class ElementImpl implements SOAPElement, SOAPBodyElement {
     
     private void register(Attr newAttr) {
         if (newAttr != null) {
-            newAttr.setUserData(SAAJ_NODE, new AttrImpl(this, newAttr), null);
+            newAttr.setUserData(SOAPDocumentImpl.SAAJ_NODE, new AttrImpl(this, newAttr), null);
         }
     }
     
     private Attr find(Attr attr) {
         if (attr != null) {
-            Object soapAttr = attr.getUserData(SAAJ_NODE);
+            Object soapAttr = attr.getUserData(SOAPDocumentImpl.SAAJ_NODE);
             if (soapAttr instanceof Attr) {
                 return (Attr) soapAttr;
             }
@@ -1710,7 +1694,11 @@ public class ElementImpl implements SOAPElement, SOAPBodyElement {
 
     @Override
     public void setIdAttributeNode(Attr idAttr, boolean isId) throws DOMException {
-        element.setIdAttributeNode(idAttr, isId);
+        if (idAttr instanceof AttrImpl) {
+            element.setIdAttributeNode(((AttrImpl)idAttr).delegate, isId);
+        } else {
+            element.setIdAttributeNode(idAttr, isId);
+        }
     }
 
     @Override
