@@ -1,28 +1,33 @@
-function getIEVersion() {
-	var ua = window.navigator.userAgent;
+function IEVersione(){
+    var ua = window.navigator.userAgent;
 
-	var msie = ua.indexOf('MSIE ');
-	if (msie > 0) {
-		// IE 10 or older => return version number
-		return Number.parseInt(ua.substring(msie + 5, ua.indexOf('.', msie)), 10);
-	}
+    var msie = ua.indexOf('MSIE ');
+    if (msie > 0) {
+      // IE 10 or older => return version number
+      return parseInt(ua.substring(msie + 5, ua.indexOf('.', msie)), 10);
+    }
 
-	var trident = ua.indexOf('Trident/');
-	if (trident > 0) {
-		// IE 11 => version
-		var rv = ua.indexOf('rv:');
-		return Number.parseInt(ua.substring(rv + 3, ua.indexOf('.', rv)), 10);
-	}
+    var trident = ua.indexOf('Trident/');
+    if (trident > 0) {
+      // IE 11 => version
+      var rv = ua.indexOf('rv:');
+      return parseInt(ua.substring(rv + 3, ua.indexOf('.', rv)), 10);
+    }
 
-	var edge = ua.indexOf('Edge/');
-	if (edge > 0) {
-		// Edge (IE 12+) => version
-		return Number.parseInt(ua.substring(edge + 5, ua.indexOf('.', edge)), 10);
-	}
+    var edge = ua.indexOf('Edge/');
+    if (edge > 0) {
+      // Edge (IE 12+) => version
+      return parseInt(ua.substring(edge + 5, ua.indexOf('.', edge)), 10);
+    }
 
-	return -1;
+    return -1;
 }
 
+function isIE(){
+    return IEVersione() > -1;
+}
+
+
 function aggiungiComandoMostraPassword(inputPasswordId){
 	
 	 // Estrai gli elementi dal DOM
@@ -88,7 +93,7 @@ function aggiungiComandoMostraPassword(inputPasswordId){
       // toggle the eye slash icon
       jQuery(this).text(passwordInput.type === 'password' ? iconVisibility : iconVisibilityOff);
       var tooltip = passwordInput.type === "password" ? iconVisibilityTooltip : iconVisibilityOffTooltip;
-      jQuery(this).attr('title', tooltip);
+      jQuery(this).prop('title', tooltip);
       
     });
-}
\ No newline at end of file
+}
